package RhinoPlayground;


import java.io.PrintWriter;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ImporterTopLevel;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.debug.DebugFrame;
import org.mozilla.javascript.debug.DebuggableScript;
import org.mozilla.javascript.debug.Debugger;

public class MyDebugger implements Debugger {

	private PrintWriter out;
	
    private int callLevel = 0;
    
    public MyDebugger() {
    	out = new PrintWriter(System.err,true);
    }
    
    public DebugFrame getFrame(Context cx, DebuggableScript fnOrScript) {
        if ( !"-debugger-".equals(fnOrScript.getSourceName())) {
            out.println("getFrame: " + fnOrScript.getFunctionName());
            callLevel++;
            return new MyDebugFrame(fnOrScript);
        }
        return null;
    }

    public void handleCompilationDone(Context cx, DebuggableScript fnOrScript,
            String source) {
        out.println("Compliation done");
    }
    
    private void printInfo( String s ) {
        
        out.print( callLevel + ":" );
        
        for( int i = 0; i< callLevel ; i++ ) {
            out.print(" ");
        }
        
        out.println(s);
        
    }

    public class MyDebugFrame implements DebugFrame {
        
        private String functionName = null;
        private String sourceFileName = null;

        private int lastLine = -1;
        
        
        MyDebugFrame( DebuggableScript function ) {
            this.functionName = function.getFunctionName();
            this.sourceFileName = function.getSourceName();
        }
        
        private void afterLine() {
            if ( lastLine > 0 ) {
                printInfo( "After Line (" + sourceFileName + ":" + lastLine + ")" + functionName );
            }
        }

        public void onEnter(Context cx, Scriptable activation, Scriptable thisObj,
                Object[] args) {

            printInfo( "onEnter: this=" + prettyPrintThis(thisObj) + " function=" + functionName);
        }
        
        private String prettyPrintThis( Scriptable thisObj ) {
            
            if (thisObj instanceof NativeJavaObject ) {
                Object nativeThis = ((NativeJavaObject) thisObj).unwrap();
                return nativeThis.toString();
            }
            if ( thisObj instanceof ImporterTopLevel ) {
                return "Global";
            }
            return Context.toString(thisObj);
        }

        public void onExceptionThrown(Context cx, Throwable ex) {
            printInfo("Exception thrown: " + ex.getMessage() );
            ex.printStackTrace(out);
        }

        public void onExit(Context cx, boolean byThrow, Object resultOrException) {
            afterLine();
            printInfo("onExit:" + " function=" + functionName);
            callLevel--;
        }

        public void onLineChange(Context cx, int lineNumber) {
            afterLine();
            lastLine = lineNumber;
        }
        
    }

}
